from matplotlib import pyplot as plt
import matplotlib.cm as cm
from  matplotlib.colors import Normalize as Normalize
from numpy import median, std
from mpl_toolkits.mplot3d import Axes3D
##from scipy.ndimage.filters import median_filter

##draw picture
def draw_pic(Data, Stars, Stars2):
    med=median(Data)
    stdv=std(Data)
    plt.imshow(Data, cmap=cm.Greys_r, aspect='equal',
                             norm= Normalize(vmin=med-stdv*2, vmax=med+stdv*5), interpolation='nearest')
    if Stars!=None:
        plt.scatter(Stars[:,0], Stars[:,1], marker = 'o', facecolors='none', edgecolors='r')
    if Stars2!=None:
        plt.scatter(Stars2[:,0], Stars2[:,1], marker = 'o', facecolors='none', edgecolors='b')
    plt.show()

##draw_astrometry errors
def draw_astrometry(mag, X_Y, diff):
    ##Ra error vs mag
    plt.plot(mag, diff[:, 0], 'bo')
    plt.ylabel('RA, (O-C)*cos(DEC)', fontsize=15)
    plt.xlabel('Vmag, APASS', fontsize=15)
    plt.title('Ra error vs Vmag')
    plt.grid()
    plt.show()

    ##De error vs mag
    plt.plot(mag, diff[:, 1], 'bo')
    plt.ylabel('DEC, (O-C)', fontsize=15)
    plt.xlabel('Vmag, APASS', fontsize=15)
    plt.title('DEC error vs Vmag')
    plt.grid()
    plt.show()

    ##Ra error vs X
    plt.plot(X_Y[:, 0], diff[:, 0], 'bo')
    plt.ylabel('RA, (O-C)', fontsize=15)
    plt.xlabel('X (pix)', fontsize=15)
    plt.title('Ra error vs X')
    plt.grid()
    plt.show()

    ##Ra error vs Y
    plt.plot(X_Y[:, 1], diff[:, 0], 'bo')
    plt.ylabel('RA, (O-C)', fontsize=15)
    plt.xlabel('Y (pix)', fontsize=15)
    plt.title('Ra error vs Y')
    plt.grid()
    plt.show()

    ##De error vs X
    plt.plot(X_Y[:, 0], diff[:, 1], 'bo')
    plt.ylabel('DEC, (O-C)', fontsize=15)
    plt.xlabel('X (pix)', fontsize=15)
    plt.title('DEC error vs X')
    plt.grid()
    plt.show()

    ##De error vs Y
    plt.plot(X_Y[:, 1], diff[:, 1], 'bo')
    plt.ylabel('DEC, (O-C)', fontsize=15)
    plt.xlabel('X (pix)', fontsize=15)
    plt.title('DEC error vs Y')
    plt.grid()
    plt.show()

##plot 3D O-C vs XY
def plot3D(X, Y, Z):
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')
##    Z = median_filter(Z, 3)
    ax.scatter(X, Y, Z, c='r', marker='.')
##    ax.plot_surface(X, Y, Z, cmap=cm.coolwarm)
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    ax.set_zlabel('O-C')
    plt.show()

##analize photometry results
def draw_photometry(Stars_XY, Stars_Mag, Colors, dPSF_mag, Obj_XY, Obj_mag, Obj_Color):
    ##delta mag vs X
    plt.plot(Stars_XY[:,0], dPSF_mag, 'bo')
    plt.plot(Obj_XY[0][0], 0, 'ro')
    plt.ylabel('O-C, Mag', fontsize=15)
    plt.xlabel('X (pix)', fontsize=15)
    plt.grid()
    plt.show()

    ##delta mag vs Y
    plt.plot(Stars_XY[:,1], dPSF_mag, 'bo')
    plt.plot(Obj_XY[0][1], 0, 'ro')
    plt.ylabel('O-C, Mag', fontsize=15)
    plt.xlabel('Y (pix)', fontsize=15)
    plt.grid()
    plt.show()

    ##delta mag vs mag
    plt.plot(Stars_Mag, dPSF_mag, 'bo')
    plt.plot(Obj_mag, 0, 'ro')
    plt.ylabel('O-C, Mag', fontsize=15)
    plt.xlabel('Mag', fontsize=15)
    plt.grid()
    plt.show()

    ##delta mag vs color
    plt.plot(Colors, dPSF_mag, 'bo')
    plt.plot(Obj_Color, 0, 'ro')
    plt.ylabel('O-C, Mag', fontsize=15)
    plt.xlabel('(B-V)', fontsize=15)
    plt.grid()
    plt.show()
    

