import pyfits
import numpy as np
import math
from scipy import stats
from scipy import optimize

################################################################
##global variables
PSF_model = []

##################################################################
##2D moffat phot
def D2_moffat(A, F, x0, y0):
    B = PSF_model[0]
    C = PSF_model[1]
    D = PSF_model[2]
    E = PSF_model[3]
    return lambda y,x:A*(1 + ((x-x0)*B)**2+((y-y0)*C)**2+((x-x0)*(y-y0)*(D**2)))**(-E)+F                   

def D2_moffat_phot(ROI, A, x_coo, y_coo, FWHM, Gain, Rnoise):
    x0=x_coo - math.floor(x_coo)+ROI.shape[1]/2
    y0=y_coo - math.floor(y_coo)+ROI.shape[0]/2
    
    ##get background
    F = PSF_model[4]
    
    #recomputer center of star
    params = (A, F, x0, y0)
    errorfunction = lambda p: np.ravel(D2_moffat(*p)(*np.indices(ROI.shape)) - ROI)
    p, success = optimize.leastsq(errorfunction, params, maxfev=1000, ftol=0.05)
    PSF_Sky = p[1]
    p[1] = 0
    p[0] = 1
    PSF = D2_moffat(*p)(*np.indices(ROI.shape))     #PSF function with height=1
    
    ##PSF
    weight_n = PSF/np.sum(PSF)                      #normalized PSF
    weighted_flux = (ROI-PSF_Sky)*weight_n
    psf_flux_n = np.sum(weighted_flux)
    psf_sum_n =  np.sum(weight_n**2)
    PSF_flux=psf_flux_n/psf_sum_n
    
    Noise=np.zeros_like(ROI)
    Noise.fill(PSF_Sky*Gain+Rnoise**2)
    Noise = np.sum(Noise*weight_n)/psf_sum_n

    PSF_mag = 25-2.5*np.log10(PSF_flux)
    PSF_Merr = 1.0857*math.sqrt(PSF_flux*Gain + Noise)/(PSF_flux*Gain)
    PSF_SN = 1.0857/PSF_Merr
    ##end PSF
    
    return (p[2]+ math.floor(x_coo)-(ROI.shape[1]/2), p[3]+ math.floor(y_coo)-(ROI.shape[0]/2), \
            PSF_flux, PSF_SN, PSF_Sky, \
            PSF_mag, PSF_Merr)

####################################################################
def PSF_phot(Data, XY, PSF, FWHM, Gain, Rnoise, SN, Saturation):
    global PSF_model
    PSF_model=PSF
    
    ##outputs empty lists
    coo=[]
    shift=[]
    mag=[]
    err=[]
    flux=[]
    sn=[]
    sky = []
    
    for ii in range(0, len(XY)):
        x_coo = XY[ii, 0] 
        y_coo = XY[ii, 1] 
        _coo = (0, 0)
        _shift = (0, 0)
        _mag = 0
        _err = 0
        _flux= 0
        _sn = 0
        _sky = 0
     
        ##check maximum level for linearity
        R1 = np.ceil(FWHM)   ##estimation of aperture radii
        Max = np.amax(Data[int(y_coo-R1):int(y_coo+R1), int(x_coo-R1):int(x_coo+R1)])
        if Max<=Saturation:
            ##copy subarray
            R3 = np.ceil(FWHM*3) ##sky annulus outer radii
            ROI =  np.copy(Data[int(y_coo-R3):int(y_coo+R3), int(x_coo-R3):int(x_coo+R3)])
            try:
                param = D2_moffat_phot(ROI, Max, x_coo, y_coo, FWHM, Gain, Rnoise)
                if param[3]>SN:
                    _coo =   (np.around(param[0],3),         np.around(param[1],3))
                    _shift = (np.around(x_coo - param[0],3), np.around(y_coo - param[1],3))
                    ##PSF_flux, PSF_SN, PSF_Sky
                    _flux= np.around(param[2], 1)
                    _sn =  np.around(param[3], 1)
                    _sky =  np.around(param[4], 1)  
                    _mag  = np.around(param[5], 4)
                    _err = np.around(param[6], 4)

            except:
                print ('#', ii, 'photomety failed')
                pass
        else:
            print ('#', ii, ' star too bright')
                
        ##append 0 if photometry failed    
        coo.append(_coo)
        shift.append(_shift)
        flux.append(_flux)
        sn.append(_sn)
        sky.append(_sky)
        mag.append(_mag)
        err.append(_err)
        
    coo = np.array(coo)
    shift = np.array(shift)
    flux = np.array(flux)
    sn = np.array(sn)
    sky= np.array(sky)
    mag = np.array(mag)
    err = np.array(err)
    
    return coo, shift, flux, sn, sky, mag, err

################################################################################
