##import standart Python modules
import os
from datetime import datetime
import numpy as np

##import my modules
from logwriter import write2log
from DB_querys import Obj_RaDec
from DB_querys import get_URAT
from get_PSF import get_PSF
from PSF2FWHM import PSF2FWHM
from PSF_phot import PSF_phot
from Astrometry import astrometry_table
from Cat_Phot import get_Phot_Catalog
from trender import trender
from Phot_Solution import Phot_Solution

##import my modules for visualisation
from Draw_Pic import draw_pic
from Draw_Pic import draw_astrometry
from Draw_Pic import plot3D
from Draw_Pic import draw_photometry

##import astropy modules
import astropy.wcs as wcs
import astropy.io.fits as fits

##disable warnings
import warnings
warnings.simplefilter("ignore")

######################################################################
##settings
object_name = 'header'              ##set object name from Horizons or use name from Header['OBJECT']
use_horizons = 'yes'                 ##use coordinates from Horizons or set manually ('no')
Ra_man = 301.56439                 ##set object coordinates manually
Dec_man = 44.45688                  ##
Cone_Search = 0.35                 ##radius of cone search for URAT
Sub_Array_length = 200              ##lenghth of subarray of nearest stars for astrometry and photometry

##set parameters for photometry
Saturation = 65000
SN = 5.0
FWHM_e = 3.0
Gain = 0.8
Rnoise = 8
SN = 5
##Obj_Color = 0.557
Obj_Color = 0.65                ##default color for asteroid

##set paths
##Path2Raw   = 'D:/Work/Python/SBG/pipeline/Raw/2017-09-02'           ##folder for raw data copy
##Path2Data  = 'D:/Work/Python/SBG/pipeline/Data/asteroid/e'                     ##directory with images
Path2Data  = 'D:/Work/Python/SBG/pipeline/Data'
##Path2Clbr  = 'D:/Work/Python/SBG/pipeline/Calibrations'             ##directory with darks and flats
Path2Temp  = 'D:/Work/Python/SBG/pipeline/Temp'                     ##directory with temp files
Path2Log   = 'D:/Work/Python/SBG/pipeline/Log'                      ##directory with log files


view='no'                           ##show pictures?
Use_Detrend_RaDec = 'no'           ##use detrending for RaDec?
Use_Colors = 0.15                   ##use stars with same colors for photometric solution?
##Use_Colors = 0 ##don't use colors

######################################################################
##make directory logfile
if not os.path.exists(Path2Log):
    os.makedirs(Path2Log)
   
##open logfile
log_name = Path2Log + '/' + 'Allmetry_' + datetime.utcnow().strftime("%Y-%m-%dT%H-%M-%S") + '.log'
##write2log(log_name, 'Path2Raw: ' + Path2Raw)
##write2log(log_name, 'Path2Clbr: ' + Path2Clbr)
write2log(log_name, 'Path2Temp: ' + Path2Temp)
write2log(log_name, 'Path2Data: ' + Path2Data)
write2log(log_name, 'saturation = ' + str(Saturation))
write2log(log_name, 'SN = ' + str(SN))
write2log(log_name, 'FWHM_e = ' + str(FWHM_e))
write2log(log_name, 'object_name = ' + object_name)
write2log(log_name, 'use_horizons = ' + use_horizons)
write2log(log_name, 'Ra_man = ' + str(Ra_man))
write2log(log_name, 'Dec_man = ' + str(Dec_man))

##make directory for tempfiles
if not os.path.exists(Path2Temp):
    os.makedirs(Path2Temp)

####for all files in Data
file_list = []
dir_content = os.listdir(Path2Data)

#create list of fits-files
for ii in range (0,len(dir_content)):
    if dir_content[ii].count('.fit') or dir_content[ii].count('.fts') or dir_content[ii].count('.fits'):
        file_list.append(Path2Data +'/'+ dir_content[ii])
        
counter = len(file_list)
if counter>0:
    ##open result file and write header
    df=open(Path2Data + '/' + 'result.txt', 'w')
    df.write('file' + '\t' + 'Object' + '\t' + 'JD' + '\t' + 'Init_Ra' + '\t' 'Init_De' + '\t' + 'Init_X' + '\t' 'Init_Y' + '\t' + \
             'NearStar' + '\t' + 'NS_mag' + '\t' + 'NPSF' + '\t' + 'FWHM' + '\t' + 'SKY' + '\t' + \
             '<RA>' + '\t' + '<DEC>' + '\t' + 'STD(RA)' + '\t' + 'STD(DEC)' + '\t' + 'Obj_RA' + '\t' + 'Obj_DEC' + '\t' + \
             'O-C_RA' + '\t' + 'O-C_DEC' + '\t' + \
             'PSF_mag'+ '\t' + 'Z-point'+ '\t' + 'X_trend'+ '\t' + 'Y_trend' + '\t' + 'C_trend' + '\t' + 'PSF_err' + '\n' )
    
    Object_RA_DEC_prev = [[0, 0]]
    Object_X_Y_Est_prev = [[0, 0]]
    #create lists
    for ii in range (0, 1): #len(file_list)):
        counter = counter-1
        print('Frames: ' + str(counter))
        file_name = file_list[ii]
        df.write(file_name.split('/')[-1] + '\t') ##write name to result
        print(file_name.split('/')[-1])
        
        ##read file, copy data and header
        Data = fits.getdata(file_name)
        Header = fits.getheader(file_name)
        JD = Header['JD'] + Header['EXPTIME']/(2*24*3600)  ##
##        JD = Header['JD_MIDDL']                               ##for IRAF reduction
        Filter = Header['FILTER']
        if object_name == 'header':
            Object = Header['OBJECT']
        else:
            Object = object_name
        df.write(Object+'\t')                           ##write Object name
        df.write(format('%.5f' % (JD-2450000))+'\t')    ##write JD to result

        ##make WCS object
        w = wcs.WCS(Header)       
          
        ##get Ra Dec of object from Horizons
        Object_RA_DEC_Est = Obj_RaDec(use_horizons, Object, JD, Ra_man, Dec_man)
        print('Object RaDec from Horizons (deg): %0.6f, %0.6f' % (Object_RA_DEC_Est[0][0], Object_RA_DEC_Est[0][1]))
        df.write(format('%.6f' % Object_RA_DEC_Est[0][0]) + '\t' + format('%.6f' % Object_RA_DEC_Est[0][1]) + '\t') ##write Horizons coordinate of object
        
        ##calculate XY cordinate of object
        Object_X_Y_Est = w.wcs_world2pix(Object_RA_DEC_Est, 1)
        print('Object XY from Horizons (pix): %0.2f, %0.2f' % (Object_X_Y_Est[0][0], Object_X_Y_Est[0][1]))
        df.write(format('%.2f' % Object_X_Y_Est[0][0]) + '\t' + format('%.2f' % Object_X_Y_Est[0][1]) + '\t') ##write XY coordinate of object

        ##check frame edge
        if (Object_X_Y_Est[0][0]>100 and Object_X_Y_Est[0][1]>100 and  \
                Object_X_Y_Est[0][0]<Data.shape[1]-100 and Object_X_Y_Est[0][1]<Data.shape[0]-100):

            ##check XY shift
            dXY = [Object_X_Y_Est_prev[0][0] - Object_X_Y_Est[0][0] , Object_X_Y_Est_prev[0][1] - Object_X_Y_Est[0][1]]
            dXY = np.sqrt(dXY[0]**2 + dXY[1]**2)
            
            ##add here checking of object shift and needs of new downloadin of URAT!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            dRADEC = [Object_RA_DEC_prev[0][0] - Object_RA_DEC_Est[0][0] , Object_RA_DEC_prev[0][1] - Object_RA_DEC_Est[0][1]]
            dRADEC = np.sqrt((dRADEC[0]*np.cos(Object_RA_DEC_Est[0][1] / (180/np.pi)))**2 + dRADEC[1]**2)
            Object_RA_DEC_prev = Object_RA_DEC_Est
            if dRADEC>Cone_Search*0.25 or dXY>min(Data.shape[0], Data.shape[1])/10:
                print("Wait new catalog")
                ##get data from Vizier, URAT1 = I/329 catalog = APASS DR8 + USNO/URAT observations
                ##parameters: RA, DEC, Radius, Row limit.
                ##Catalog: RA2000, DEC2000, B, e_B, V, e_V, g, e_g, r, e_r, i, e_i, pmRA, pmDEC
                Catalog = get_URAT(Object_RA_DEC_Est[0][0], Object_RA_DEC_Est[0][1], Cone_Search, 1000) ##0.35
                print('Catalog lenght: ' + str(len(Catalog)))
            else:
                print("Old catalog in use")

            ##trim stars outside frame
            XY_2000 = w.wcs_world2pix(Catalog[:, 0:2], 0)        
            Index = np.where((XY_2000[:,0]>50) & (XY_2000[:,0]<Data.shape[1]-50) & (XY_2000[:,1]>50) & (XY_2000[:,1]<Data.shape[0]-50))[0]
            Cat_local = Catalog[Index]
            XY_2000 = XY_2000[Index]
            print('Stars on frame: ' + str(len(Cat_local)))
            
            ##get actual XY shift of stars by proper motion
            ##proper motion is small, so we use simple linear WCS solution from header for calculation of XY shift
            PM = Cat_local[:, 12:14]   
            PM[:, 0] = PM[:, 0] / (np.cos(Cat_local[:, 1] / (180/np.pi)))
            dT = (JD - 2451545)/365.25
            PM = PM *dT / (1000*3600)
            Cat_RaDec_Now = Cat_local[:, 0:2] + PM
            XY_corr = w.wcs_world2pix(Cat_RaDec_Now[:, 0:2], 0) - XY_2000
            ##XY_corr = difference in XY positions of stars between NOW and 2000
            ##we use 2MASS-PSC with Epoch=2000 for precise astrometry and must subtract XY_corr from measured XY

            ##calculate distance from object to all stars from catalog
            Distance = XY_2000-Object_X_Y_Est
            Distance = np.sqrt(Distance[:, 0]*Distance[:, 0] + Distance[:, 1]*Distance[:, 1])

            ##sort catalog by distance
            Index = np.argsort(Distance)
            Distance = Distance[Index]
            XY_2000 = XY_2000[Index]
            XY_corr = XY_corr[Index]
            Cat_local = Cat_local[Index]
            
            ##check distance from object to nearest stars!
            print('Nearest star: %0.2f(pix), Vmag: %0.2f' % (Distance[0], Cat_local[0, 4]))
            df.write(format('%.2f' % Distance[0]) + '\t' + format('%.2f' % Cat_local[0, 4]) + '\t') ##write distance to nearest star and it's magnitude

            ##make subarray of stars near object
            Len = min(Sub_Array_length, len(Distance)) ##200
            Distance = Distance[0:Len]
            XY_2000 = XY_2000[0:Len]
            XY_corr = XY_corr[0:Len]
            Cat_local = Cat_local[0:Len]
            print('Stars in subarray: ' + str(Len) + ' Max: ' + str(Sub_Array_length))
            
            ##make subarray of PSF stars
            Index = np.where((Cat_local[:, 4]>12) & (Cat_local[:, 4]<15))[0]
            PSF_stars = XY_2000[Index]            
            print(str(len(PSF_stars))+' stars use for PSF model')
            df.write(format('%d' % len(PSF_stars)) + '\t')          ##write number of PSF stars
            ##making of PSF model
            PSF_model = get_PSF(Data, PSF_stars, FWHM_e)
            FWHM1, FWHM2, Phi, Sky = PSF2FWHM(PSF_model)
            ##print FWHM and sky
            print('FWHM1={0:.2f}'.format(FWHM1), '\t', 'FWHM2={0:.2f}'.format(FWHM2), '\t',
                    'Phi={0:.2f}'.format(Phi*180/np.pi), '\t', 'Sky={0:.1f}'.format(Sky))

            FWHM = max(FWHM1, FWHM2)
            df.write(format('%.1f' % FWHM) + '\t' + format('%.1f' % Sky) + '\t') ##write FWHM and sky
            
            ##PSF photometry of stars from full list       
            X_Y_Now, shift, PSF_flux, PSF_sn, PSF_Sky, Mags, Merrs = \
                     PSF_phot(Data, XY_2000, PSF_model, FWHM, Gain, Rnoise, SN, Saturation) ##return coo, shift, flux, sn, sky, mag, mag_err

            ##delete stars with zero magnitude
            Index = np.where(Mags!=0)[0]
            X_Y_Now = X_Y_Now[Index]
            shift = shift[Index]
            Mags = Mags [Index]
            Merrs = Merrs [Index]
            PSF_flux = PSF_flux [Index]
            PSF_sn = PSF_sn [Index]
            PSF_Sky = PSF_Sky[Index]
            XY_2000 = XY_2000[Index]
            Cat_local = Cat_local[Index]
            XY_corr = XY_corr[Index]
            Distance = Distance[Index]
            
            ##delete stars with abnormal shift
            Ra = np.median(shift[:, 0])
            RaMin = Ra - 3* np.std(shift[:, 0])
            RaMax = Ra + 3* np.std(shift[:, 0])
            De = np.median(shift[:, 1])
            DeMin = De - 3* np.std(shift[:, 1])
            DeMax = De + 3*np.std(shift[:, 1])
            
            Index = np.where((shift[:, 1]<DeMax) & (shift[:, 1]>DeMin) & (shift[:, 0]<RaMax) & (shift[:, 0]>RaMin))[0]
            X_Y_Now = X_Y_Now[Index]
            shift = shift[Index]
            Mags = Mags [Index]
            Merrs = Merrs [Index]
            PSF_flux = PSF_flux [Index]
            PSF_sn = PSF_sn [Index]
            PSF_Sky = PSF_Sky[Index]
            XY_2000 = XY_2000[Index]
            Cat_local = Cat_local[Index]
            XY_corr = XY_corr[Index]
            Distance = Distance[Index]

            print('Stars after photometry: ' + str(len(Mags)))
                   
            ##apply proper motion correction
            X_Y = X_Y_Now - XY_corr
            ##sort by magnitude
            Index = Mags.argsort()
            X_Y = X_Y[Index]
            X_Y_Now = X_Y_Now[Index]
            shift = shift[Index]
            flux = PSF_flux [Index]
            PSF_sn = PSF_sn [Index]
            PSF_Sky = PSF_Sky[Index]
            XY_2000 = XY_2000[Index]
            Cat_local = Cat_local[Index]
            XY_corr = XY_corr[Index]
            Mags = Mags [Index]
            Merrs = Merrs [Index]
            Distance = Distance[Index]
             
            ##make fits-table with XY coordinate
            X_offset = np.min(X_Y[:, 0])
            Y_offset = np.min(X_Y[:, 1])
            c1 = fits.Column(name='XIMAGE', format='E', array=X_Y[:, 0]-X_offset)
            c2 = fits.Column(name='YIMAGE', format='E', array=X_Y[:, 1]-Y_offset)
            c3 = fits.Column(name='FLUX',   format='E', array=Mags)
            tbhdu = fits.BinTableHDU.from_columns([c1, c2, c3])
            tbhdu.header["IMAGEW"] = np.max(X_Y[:, 0]) - X_offset
            tbhdu.header["IMAGEH"] = np.max(X_Y[:, 1]) - Y_offset
            tbhdu.writeto(Path2Temp + '/' + 'temp.fits', clobber = True)

            ##astrometry solution for region around object
            Success = astrometry_table(Path2Temp + '/' + 'temp.fits', Object_RA_DEC_Est[0][0], Object_RA_DEC_Est[0][1], 1)

            if Success:
                Header = fits.getheader(Path2Temp + '/' + 'temp_WCS.fits')
                ##make WCS object
                w = wcs.WCS(Header)
                ##calculate RaDec for measured stars
                Stars_RA_DEC = w.all_pix2world(X_Y-[X_offset, Y_offset], 1)
                
                ##photometry and astrometry of object
                Obj_XY, Obj_shift,  Obj_PSF_flux, Obj_PSF_sn, Obj_PSF_Sky, Obj_Mags, Obj_Merrs = \
                        PSF_phot(Data, Object_X_Y_Est, PSF_model, FWHM, Gain, Rnoise, SN, 65000)

                ##calculate object Eq coo
                Object_RA_DEC = w.all_pix2world(Obj_XY-[X_offset, Y_offset], 1)

                if Use_Detrend_RaDec == 'yes':                
                    ##detrend Ra by X
                    trend, Obj_trend = trender(X_Y[:,0], (Stars_RA_DEC[:, 0]-Cat_local[:, 0]), Obj_XY[0][0], 1, 'no', 'X')
                    Stars_RA_DEC[:, 0] = Stars_RA_DEC[:, 0]-trend
                    Object_RA_DEC[0][0] = Object_RA_DEC[0][0] - Obj_trend
                    
                    ##detrend Ra by Y
                    trend, Obj_trend = trender(X_Y[:,1], (Stars_RA_DEC[:, 0]-Cat_local[:, 0]), Obj_XY[0][0], 1, 'no', 'Y')
                    Stars_RA_DEC[:, 0] = Stars_RA_DEC[:, 0]-trend
                    Object_RA_DEC[0][0] = Object_RA_DEC[0][0]-Obj_trend
                    
                    ##detrend Dec by X
                    trend, Obj_trend = trender(X_Y[:,0], (Stars_RA_DEC[:, 1]-Cat_local[:, 1]), Obj_XY[0][1], 1, 'no', 'X')
                    Stars_RA_DEC[:, 1] = Stars_RA_DEC[:, 1]-trend
                    Object_RA_DEC[0][1] = Object_RA_DEC[0][1]-Obj_trend
                    
                    ##detrend Dec by Y
                    trend, Obj_trend = trender(X_Y[:,1], (Stars_RA_DEC[:, 1]-Cat_local[:, 1]), Obj_XY[0][1], 1, 'no', 'Y')
                    Stars_RA_DEC[:, 1] = Stars_RA_DEC[:, 1]-trend
                    Object_RA_DEC[0][1] = Object_RA_DEC[0][1]-Obj_trend

                ##caclulate O-C
                diff = (Stars_RA_DEC-Cat_local[:, 0:2])*3600
                diff[:, 0] = diff[:, 0] * np.cos(Cat_local[:, 1]/(180/np.pi))

                ##graph analize astrometry results
                if view == 'yes':
                    draw_astrometry(Cat_local[:, 4], X_Y, diff)
                    
                print('Median O-C for RA (arcsec): %0.3f' % np.median(diff[:, 0]))
                print('Median O-C for DEC (arcsec): %0.3f' % np.median(diff[:, 1]))
                df.write(format('%.3f' % np.median(diff[:, 0])) + '\t' + format('%.3f' % np.median(diff[:, 1])) + '\t') ##write info about astrometry solution
                print('Sigma O-C for RA (arcsec): %0.3f' % np.std(diff[:, 0]))
                print('Sigma O-C for DEC (arcsec): %0.3f' % np.std(diff[:, 1]))
                df.write(format('%.3f' % np.std(diff[:, 0])) + '\t' + format('%.3f' % np.std(diff[:, 1])) + '\t')    ##write info about astrometry solution

                print('Object RaDec (deg): %0.5f, %0.5f' % (Object_RA_DEC[0][0], Object_RA_DEC[0][1]))
                O_C = (Object_RA_DEC[0] - Object_RA_DEC_Est[0])*3600            
                O_C[0] = O_C[0] * np.cos(Object_RA_DEC[0][1]/(180/np.pi))
                print('Object O-C, Ra, Dec (arcsec): %0.2f, %0.2f' % (O_C[0], O_C[1]))
                df.write(format('%.5f' % Object_RA_DEC[0][0]) + '\t' + format('%.5f' % Object_RA_DEC[0][1]) + \
                         '\t' +format('%.2f' % O_C[0]) + '\t' +format('%.2f' % O_C[1]) + '\t')                         ##write info about astrometry of object


                ##get photometric data from catalog for "Filter": mag, error, B-V
                Cat_Phot = get_Phot_Catalog(Filter, Cat_local)

                ##select stars by colors
                if Use_Colors !=0:
                    C = np.abs(Cat_Phot[:, 2] - Obj_Color)
                    Index = np.where(C<Use_Colors)
                    Cat_Phot = Cat_Phot[Index]
                    X_Y_Now = X_Y_Now[Index]
                    Mags =Mags[Index]
                    Merrs = Merrs[Index]
                print('Stars with good colors: ' + str(len(Mags)))
                
                ############################
                ##call Phot_Solution
                Obj_mag_raw, Obj_mag_Z, Obj_mag_X, Obj_mag_Y, Obj_mag_C \
                    = Phot_Solution(Mags, Merrs, Obj_Mags, Obj_Merrs, Cat_Phot, X_Y_Now, Obj_XY, Obj_Color, view)
            
                df.write(format('%.4f' % Obj_mag_raw) + '\t') ##write raw magnitude
                df.write(format('%.4f' % Obj_mag_Z) + '\t') ##write zero point corrected magnitude
                df.write(format('%.4f' % Obj_mag_X) + '\t') ##write X corrected magnitude
                df.write(format('%.4f' % Obj_mag_Y) + '\t') ##write X corrected magnitude
                df.write(format('%.4f' % Obj_mag_C) + '\t') ##write X corrected magnitude
                df.write(format('%.4f' % Obj_Merrs) + '\t') ##write PSF_mag error
                
                print('Object Mag: %0.4f' % Obj_mag_C)
                print('Object Error: %0.4f' % Obj_Merrs)
                
##                df.write('\n')
                ##picture+object+stars
                if view == 'yes':
                    draw_pic(Data, Obj_XY, X_Y_Now)

            
        else:
            print('Object out of frame')
            df.write('\n')

        print()

    ##close result output
    df.close()
      
            
